import logging as log

log.basicConfig(level=log.WARN,
                format='%(asctime)s: %(levelname)s [%(filename)s:%(lineno)s] %(message)s',
                datefmt='%I:%M:%S %p',
                handlers=[
                    log.StreamHandler()
                ]
                )




class PetriNet:

    def __init__(self, name, places, transitions, inputs, outputs, initial_marking):
        self.__name = name
        self.__places = places
        self.__transitions = transitions
        self.__inputs = inputs
        self.__outputs = outputs
        self.__initial_marking = initial_marking
        self.__current_marking = initial_marking

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, name):
        self.__name = name

    @property
    def places(self):
        return self.__places

    @places.setter
    def places(self, places):
        self.__places = places

    @property
    def transitions(self):
        return self.__transitions

    @transitions.setter
    def transitions(self, transitions):
        self.__transitions = transitions

    @property
    def inputs(self):
        return self.__inputs

    @inputs.setter
    def inputs(self, inputs):
        self.__inputs = inputs

    @property
    def outputs(self):
        return self.__outputs

    @outputs.setter
    def outputs(self, outputs):
        self.__outputs = outputs

    @property
    def initial_marking(self):
        return self.__initial_marking

    @initial_marking.setter
    def initial_marking(self, initial_marking):
        self.__initial_marking = initial_marking

    @property
    def current_marking(self):
        return self.__current_marking

    @current_marking.setter
    def current_marking(self, current_marking):
        self.__current_marking = current_marking

    def __str__(self):
        return f'''
            Red Name: {self.__name}
                Places: {self.__places}
                Transitions: {self.__transitions}
                Inputs: {self.__inputs}
                Outputs: {self.__outputs}
                Initial Marking: {self.__initial_marking}
                Current Marking: {self.__current_marking}
        '''


from data.HandleJSON import HandleJSON
from logger.logger import log
import json

class LoadJSON:

    '''
    Clase encargada de cargar el archivo JSON
    '''

    def __init__(self, path, id_net):
        '''
        Metodo __init__ para inicializar los valores del Objeto, recibe los parametros:
        :param path: ruta del archivo a cargar
        :param id_net: id de la red a cargar
        '''
        self.__path = path
        self.__id_net = id_net
        self.__net = None

    def laod_net(self):
        '''
        Metodo encargado de cargar la red y asignarla a la propiedad self.__net
        '''
        try:
            with HandleJSON(self.__path, 'r') as document:
                data = json.load(document)
                self.__net = data[self.__id_net]
        except Exception as e:
            log.error(f'Load JSON Error: {e}')

    def load_places(self):
        '''
        Metodo encargado de cargar los lugares de la red, retorna
        :return: Diccionario con los lugares de la red
        '''
        if self.__net is None:
            self.laod_net()
        places = dict(self.__net['places'])
        return places

    def load_transitions(self):
        '''
        Metodo encargado de cargar las transiciones de la red, retorna
        :return: Lista con las transiciones de la red
        '''
        if self.__net is None:
            self.laod_net()
        transitions = list(self.__net['transitions'])
        return transitions

    def load_inputs(self):
        '''
        Metodo encargado de cargar las funciones de entrada de la red, retorna
        :return: Diccionario con las funciones de entrada de la red
        '''
        if self.__net is None:
            self.laod_net()
        inputs = dict(self.__net['inputs'])
        return inputs

    def load_outputs(self):
        '''
        Metodo encargado de cargar las funciones de salida de la red, retorna
        :return: Diccionario con las funciones de salida de la red
        '''
        if self.__net is None:
            self.laod_net()
        outputs = dict(self.__net['outputs'])
        return outputs

    def load_initial_marking(self):
        '''
        Metodo encargado de cargar la marcacion inicial de la red, retorna
        :return: Lista con la marcacion inicial de la red
        '''
        places = self.load_places()
        initial_marking = list(places.values())
        return initial_marking

    def load_name(self):
        if self.__net is None:
            self.laod_net()
        name = str(self.__net['name'])
        return name

    def load_ids(self):
        try:
            with HandleJSON(self.__path, 'r') as document:
                data = dict(json.load(document))
                return list(data.keys())
        except Exception as e:
            log.error(f'Error al cargar el archivo: {e}')


if __name__ == '__main__':
    loadJson = LoadJSON('../documentosJSON.json', '02')
    print(loadJson.load_ids())



import graphviz as gr
import os
os.environ["PATH"] += os.pathsep + 'C:/Program Files (x86)/Graphviz2.38/bin'

class PetriNetDraw:
    def __init__(self, petri_net):
        self.__grph_net = gr.Digraph(petri_net.name, format='svg')

    def add_places(self, places, marking):
        self.__grph_net.attr('node', shape='circle')
        for place, mark in zip(places, marking):
            self.__grph_net.node(place, place + '\n' + str(int(mark)))

    def add_transitions(self, transitions):
        self.__grph_net.attr('node', shape='rectangle')
        for trans in transitions:
            self.__grph_net.node(trans)

    def add_functions(self, functions):
        value_functions = list(functions.values())
        functions_add = []
        for val in value_functions:
            if functions_add.count(val) == 0:
                peso = value_functions.count(val)
                self.__grph_net.edge(val[0], val[1], label=str(peso))
            functions_add.append(val)

    def draw_petri_net_with_initial_marking(self, petri_net):
        self.add_places((petri_net.places.keys()), petri_net.initial_marking)
        self.add_transitions(petri_net.transitions)
        self.add_functions(petri_net.inputs)
        self.add_functions(petri_net.outputs)
        self.__grph_net.render(directory='states-%s'%petri_net.name, view=True, filename='initial_state')

    def draw_petri_net_with_current_marking(self, petri_net):
        self.add_places((petri_net.places.keys()), petri_net.current_marking)
        self.add_transitions(petri_net.transitions)
        self.add_functions(petri_net.inputs)
        self.add_functions(petri_net.outputs)
        self.__grph_net.render(directory='states-%s' % petri_net.name, view=True, filename='current_state')


import numpy as np

class PetriNetDynamism:

    @staticmethod
    def create_matrix(functions, places, transitions, type):
        matrix = np.zeros(shape=(len(transitions), len(places)))
        row = 0 if type else 1
        col = 1 if type else 0
        for key, function in functions.items():
            place_ind = places.index(function[row])
            transitions_ind = transitions.index(function[col])
            matrix[transitions_ind][place_ind] += 1
        return matrix

    @staticmethod
    def get_ej(transitions, trans_shoot):
        ej = np.zeros(shape=len(transitions))
        ind_trans = transitions.index(trans_shoot)
        ej[ind_trans] += 1
        return ej

    @staticmethod
    def get_ejs(transitions, shoots):
        ej = np.zeros(shape=len(transitions))
        for shoot in shoots:
            ind_trans = transitions.index(shoot[0])
            ej[ind_trans] = shoot[1]
        return ej

    @staticmethod
    def transition_enable(transitions, marking, matrix_input, trans_shoot):
        ej = PetriNetDynamism.get_ej(transitions, trans_shoot)
        array_aux = np.dot(ej, matrix_input)
        return np.all(np.less_equal(array_aux, marking))

    @staticmethod
    def transitions_enabled(transitions, marking, matrix_input):
        enabled = []
        for transition in transitions:
            if PetriNetDynamism.transition_enable(transitions, marking, matrix_input, transition):
                enabled.append(transition)
        return enabled

    @staticmethod
    def shoot_transition(marking, transitions, matrix, trans_shoot):
        ej = PetriNetDynamism.get_ej(transitions, trans_shoot)
        new_marking = marking + (np.dot(ej, matrix))
        return new_marking

    @staticmethod
    def shoot_burst(marking, transitions, matrix, trans):
        ejs = PetriNetDynamism.get_ejs(transitions, trans)
        new_marking = marking + (np.dot(ejs, matrix))
        return new_marking

    @staticmethod
    def create_matrix_d(functions, places, transitions):
        return PetriNetDynamism.create_matrix(functions[1], places, transitions, False) - PetriNetDynamism.create_matrix(functions[0], places, transitions, True)



class HandleJSON:

    '''
    Clase para cargar el archivo JSON
    '''

    def __init__(self, path, type):
        '''
        Metodo inicializador de la clase, recibe los parametros:
        :param path: Indica la ruta del archivo JSON a cargar
        :param type: Indica el tipo de tratamiento que se le dara al archivo JSON
        '''
        self.path = path
        self.type = type

    def __enter__(self):
        '''
        Metodo que se ejecuta cuando se carga el archivo, abre el archivo y lo retorna, retorna:
        :return: Objeto del archivo cargado.
        '''
        self.path = open(self.path, self.type)
        return self.path

    def __exit__(self, exc_type, exc_val, exc_tb):
        '''
        Metodo que se ejecuta al momento en que termina la carga del archivo, cierra el archivo si está abierto, recibe los parametros:
        :param exc_type: Tipo de excepcion, si la hay
        :param exc_val: Valor de la expecion, si la hay
        :param exc_tb: Errores, si los hay
        '''
        if self.path:
            self.path.close()


from data.LoadJSON import LoadJSON
from net.PetriNet import PetriNet
from exception.HapyException import HapyException
from dynamism.PetriNetDynamism import PetriNetDynamism as pnd
from dynamism.PetriNetDraw import PetriNetDraw
import os

def create_petri_net(path, id_net):
    if os.path.isfile(path):
        lj = LoadJSON(path, id_net)
        if id_net in lj.load_ids():
            name = lj.load_name()
            places = lj.load_places()
            transitions = lj.load_transitions()
            inputs = lj.load_inputs()
            outputs = lj.load_outputs()
            initial_marking = lj.load_initial_marking()
            return PetriNet(name, places, transitions, inputs, outputs, initial_marking)
        else:
            raise HapyException('El id enviado no se encuentra en los datos del JSON...')
    else:
        raise HapyException('La ruta del archivo enviado no existe...')

def transitions_enabled(petri_net):
    transitions = petri_net.transitions
    places_dict = petri_net.places
    places = list(places_dict.keys())
    marking = petri_net.current_marking
    inputs = petri_net.inputs
    matrix_input = pnd.create_matrix(inputs, places, transitions, True)
    return pnd.transitions_enabled(transitions, marking, matrix_input)

def transition_enable(petri_net, transition):
    transitions = petri_net.transitions
    marking = petri_net.current_marking
    places_dict = petri_net.places
    places = list(places_dict.keys())
    inputs = petri_net.inputs
    matrix_input = pnd.create_matrix(inputs, places, transitions, True)
    if transition in transitions:
        return pnd.transition_enable(transitions, marking, matrix_input, transition)
    else:
        raise HapyException('La transicion a disparar no se encuentra establecida en la red...')

def shoot_transition(petri_net, trans_shoot):
    if transition_enable(petri_net, trans_shoot):
        transitions = petri_net.transitions
        current_marking = petri_net.current_marking
        inputs = petri_net.inputs
        outputs = petri_net.outputs
        places = list(petri_net.places.keys())
        functions = [inputs, outputs]
        matrix = pnd.create_matrix_d(functions, places, transitions)
        new_marking = pnd.shoot_transition(current_marking, transitions, matrix, trans_shoot)
        petri_net.current_marking = new_marking
    else:
        raise HapyException('La transicion a disparar no esta habilitada...')

def shoot_burst(petri_net, *args):
    transitions = petri_net.transitions
    current_marking = petri_net.current_marking
    inputs = petri_net.inputs
    outputs = petri_net.outputs
    places = list(petri_net.places.keys())
    functions = [inputs, outputs]
    matrix = pnd.create_matrix_d(functions, places, transitions)
    new_marking = pnd.shoot_burst(current_marking, transitions, matrix, args)
    petri_net.current_marking = new_marking

def draw_petri_net_with_initial_marking(petri_net):
    draw = PetriNetDraw(petri_net)
    draw.draw_petri_net_with_initial_marking(petri_net)

def draw_petri_net_with_current_marking(petri_net):
    draw = PetriNetDraw(petri_net)
    draw.draw_petri_net_with_current_marking(petri_net)

if __name__ == '__main__':
    print(create_petri_net('documentosJSON.json', '02'))


